#include "SpinningTop.h"

#include "Barycentric.h"

SpinningTop::SpinningTop(Mesh mesh, Shader shader, Vec4f position = Vec4f(0.0f, 0.0f, 0.0f, 1.0f))
    : mesh(mesh)
    , shader(shader)
    , color(1.0f, 1.0f, 1.0f)
    , position(position)
    , movingDirection(0.0f, 0.0f, 1.0f, 0.0f)
    , smoothedDirection(movingDirection)
{}



void SpinningTop::simulate(World& world){
    // Gets all the triangles of the ground:
    std::vector<Triangle>& groundTriangles = world.groundMesh.getTriangles();

    // Calculates a new moving direction by randomly rotate the
    // moving direction around the y axis:
    float random = (float(rand()) / float(RAND_MAX) * 2.f - 1.f) * 0.1f;
    movingDirection = Mat4f::rotationY(random) * movingDirection;
    smoothedDirection = smoothedDirection * 0.98f + movingDirection * 0.02f;

    // Applies the moving in XZ direction:
    position.x += movingDirection.x * speed;
    position.z += movingDirection.z * speed;

    /*
     * *TODO* (Exercise 3): Go through all triangles and...
     *
     *  (a) find the triangle which is exactly below this SpinningTop.
     *      You should to this by calculating the barycentric coordinate,
     *      but note, that you have to project the plane and the position
     *      to the same plane beforehand. You can use the XZ plane as common
     *      plane here (i.e. set y=0).
     *
     *  (b) use the barycentric interpolation to calculate the height of
     *      this spinning top, add 0.25f to it (because the lower tip of
     *      the SpinningTop mesh is at y=-0.25 in model space) and set it
     *      to the y-value of the positon attribute.
     *
     *  (c) use the barycentric interpolation to calculate the color of
     *      this spinning top (so that it is identical with the color the
     *      triangle below the tip) - see the 'color' attribute of this
     *      class. Note that you can use the multiplication operator of
     *      the Vec4f class, but consider that the Vector must be on the
     *      left side of the operation (i.e. Vec4f * float).
     *
     *  (d) Wepdate the speed according to the following formula:
     *      speed <- speed + triangleNormal.dot(movingDirection) * 0.0002f.
     *      (In this example, all vertices share the same normal!)
     *
     *      (Note that this is far away from being realistic, but creates a
     *      little bit of 'gravity' impression :-)).
     */

    for(Triangle& triangle : groundTriangles){
        // (a)
        bool isInTriangle = false; // Teste ob das Dreieck mit der Kreiselspitze kollidiert
        if(isInTriangle){
            // (b)
            float height = 0; // Interpoliere die Höhe der Eckpunkte d. Dreiecks
            position.y = height + 0.25f;

            // (c)
            color = Vec4f(0,0,0); // Interpoliere die Farbe der Eckpunkte d. Dreiecks

            // (d)
            // speed = speed + ...; // Bewegungsrichtung entspr. der Formel anpassen
        }
    }

    // Calculate the final model matrix by using the position
    // and the movingDirection
    {
        // Second column of the model matrix (we want the
        // y axis to point always straight to the top):
        Vec4f yAxis = Vec4f(0.0f, 1.0f, 0.0f, 0.0f);

        // Third column of the model matrix:
        Vec4f zAxis = smoothedDirection * (speed > 0.f ? 1.f : -1.f);
        // Vec4f zAxis = smoothedDirection * speed;

        // First column of the model matrix:
        Vec4f xAxis = yAxis.cross(zAxis);

        // Creates a model matrix where xAxis, yAxis, zAxis
        // and position are used as columns:
        modelMatrix = Mat4f(xAxis, yAxis, zAxis, position);
    }
}


void SpinningTop::render(Mat4f view, Mat4f projection){
    // Bind our shader:
    shader.bind();

    // Assume that our world mesh is already correctly transformed in the world
    // so we upload just the identity matrix:
    shader.setUniform("model", modelMatrix);

    // Upload our view matrix, which performs the transformation to view the
    // world from the 'perspective' of something, e.g. a camera or a pawn:
    shader.setUniform("view", view);

    // Upload the projection matrix (used in vertex shader):
    shader.setUniform("projection", projection);

    shader.setUniform("color", color);

    // Draw our triangle:
    mesh.render();
}

Mat4f SpinningTop::getModelMatrix(){
    return modelMatrix;
}
